"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InnerShipStatus = void 0;
const util_1 = require("@skeldjs/util");
const constant_1 = require("@skeldjs/constant");
const data_1 = require("@skeldjs/data");
const Networkable_1 = require("../Networkable");
const events_1 = require("../events");
const PlayerInfo_1 = require("../misc/PlayerInfo");
function shuffleArray(array) {
    for (let i = array.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        const temp = array[i];
        array[i] = array[j];
        array[j] = temp;
    }
}
class InnerShipStatus extends Networkable_1.Networkable {
    constructor(room, spawnType, netid, ownerid, flags, data) {
        super(room, spawnType, netid, ownerid, flags, data);
        if (!this.systems) {
            this.systems = new Map;
            this.Setup();
        }
        this.spawnRadius = 1;
        this.initialSpawnCenter = util_1.Vector2.null;
        this.meetingSpawnCenter = util_1.Vector2.null;
    }
    get owner() {
        return super.owner;
    }
    // eslint-disable-next-line @typescript-eslint/no-empty-function
    Setup() { }
    Deserialize(reader, spawn = false) {
        if (!this.systems) {
            this.systems = new Map;
            this.Setup();
        }
        while (reader.left) {
            const [tag, mreader] = reader.message();
            const system = this.systems.get(tag);
            if (system) {
                system.Deserialize(mreader, spawn);
            }
        }
    }
    /* eslint-disable-next-line */
    Serialize(writer, spawn = false) {
        for (const [, system] of this.systems) {
            if (system.dirty) {
                writer.begin(system.systemType);
                system.Serialize(writer, spawn);
                writer.end();
                system.dirty = false;
            }
        }
        this.dirtyBit = 0;
        return true;
    }
    async HandleRpc(rpc) {
        switch (rpc.messageTag) {
            case constant_1.RpcMessageTag.CloseDoorsOfType:
                await this._handleCloseDoorsOfType(rpc);
                break;
            case constant_1.RpcMessageTag.RepairSystem:
                await this._handleRepairSystem(rpc);
                break;
        }
    }
    async _handleCloseDoorsOfType(rpc) {
        const doors = this.systems.get(constant_1.SystemType.Doors);
        const doorsInRoom = this.getDoorsInRoom(rpc.systemId);
        for (const doorId of doorsInRoom) {
            doors.closeDoor(doorId);
        }
    }
    FixedUpdate(delta) {
        for (const [, system] of this.systems) {
            system.Detoriorate(delta);
        }
    }
    async _handleRepairSystem(rpc) {
        const system = this.systems.get(rpc.systemId);
        const player = this.room.getPlayerByNetId(rpc.netId);
        if (system && player) {
            await system.HandleRepair(player, rpc.amount, rpc);
        }
    }
    /**
     * Randomly select players to be the Impostor. Called after a game is started
     * and emits a {@link RoomSelectImpostorsEvent} which can be used to alter the
     * results of this function.
     */
    async selectImpostors() {
        var _a;
        const available = [...this.room.players.values()].filter((player) => player.info && !player.info.isDisconnected && !player.info.isDead);
        const max = available.length < 7 ? 1 : available.length < 9 ? 2 : 3;
        const impostors = [];
        for (let i = 0; i < Math.min(this.room.settings.numImpostors, max); i++) {
            const random = ~~(Math.random() * available.length);
            impostors.push(available[random]);
            available.splice(random, 1);
        }
        const ev = await this.emit(new events_1.RoomSelectImpostorsEvent(this.room, impostors));
        if (!ev.canceled && ((_a = this.room.host) === null || _a === void 0 ? void 0 : _a.control)) {
            await this.room.host.control.setImpostors(ev.alteredImpostors);
        }
    }
    getTasksForMap(map) {
        switch (map) {
            case constant_1.GameMap.TheSkeld:
                return Object.values(data_1.TheSkeldTasks);
            case constant_1.GameMap.MiraHQ:
                return Object.values(data_1.MiraHQTasks);
            case constant_1.GameMap.Polus:
                return Object.values(data_1.PolusTasks);
            case constant_1.GameMap.AprilFoolsTheSkeld:
                return Object.values(data_1.TheSkeldTasks);
            case constant_1.GameMap.Airship:
                return Object.values(data_1.AirshipTasks);
        }
        return [];
    }
    addTasksFromList(start, count, tasks, usedTaskTypes, unusedTasks) {
        if (unusedTasks.length === 0) {
            return start;
        }
        let numLoops = 0; // fun fact: nodepolus calls this a "sanity check"
        let i = 0;
        while (i < count && numLoops++ < 1000) {
            if (start >= unusedTasks.length) {
                start = 0;
                shuffleArray(unusedTasks);
                if (unusedTasks.every(taskData => usedTaskTypes.has(taskData.taskType))) {
                    usedTaskTypes.clear();
                }
            }
            const task = unusedTasks[start++];
            if (usedTaskTypes.has(task.taskType)) {
                i--;
            }
            else {
                usedTaskTypes.add(task.taskType);
                tasks.push(task.index);
            }
            i++;
        }
        return start;
    }
    async assignTasks() {
        const allTasks = this.getTasksForMap(this.room.settings.map);
        const numCommon = this.room.settings.commonTasks;
        const numLong = this.room.settings.longTasks;
        const numShort = this.room.settings.shortTasks;
        const allCommon = [];
        const allLong = [];
        const allShort = [];
        for (let i = 0; i < allTasks.length; i++) {
            switch (allTasks[i].length) {
                case constant_1.TaskLength.Common:
                    allCommon.push(allTasks[i]);
                    break;
                case constant_1.TaskLength.Long:
                    allLong.push(allTasks[i]);
                    break;
                case constant_1.TaskLength.Short:
                    allShort.push(allTasks[i]);
                    break;
            }
        }
        shuffleArray(allCommon);
        shuffleArray(allLong);
        shuffleArray(allShort);
        const usedTaskTypes = new Set;
        const commonTasks = [];
        this.addTasksFromList(0, numCommon, commonTasks, usedTaskTypes, allCommon);
        let shortIdx = 0;
        let longIdx = 0;
        for (const [, player] of this.room.players) {
            if (!player.info)
                continue;
            usedTaskTypes.clear();
            const playerTasks = [...commonTasks];
            shortIdx = this.addTasksFromList(shortIdx, numShort, playerTasks, usedTaskTypes, allShort);
            longIdx = this.addTasksFromList(longIdx, numLong, playerTasks, usedTaskTypes, allLong);
            player.info.setTaskIds(playerTasks);
            player.info.setTaskStates(playerTasks.map((task, taskIdx) => {
                return new PlayerInfo_1.TaskState(taskIdx, false);
            }));
        }
    }
    getSpawnPosition(player, initialSpawn) {
        const playerId = typeof player === "number"
            ? player
            : player.playerId;
        return util_1.Vector2.up
            .rotateDeg(playerId * (360 / this.room.players.size))
            .mul(this.spawnRadius)
            .add(initialSpawn
            ? this.initialSpawnCenter
            : this.meetingSpawnCenter)
            .add(new util_1.Vector2(0, 0.3636));
    }
    spawnPlayer(player, initialSpawn) {
        var _a;
        if (player.playerId === undefined)
            return;
        (_a = player.transform) === null || _a === void 0 ? void 0 : _a.snapTo(this.getSpawnPosition(player, initialSpawn));
    }
    getDoorsInRoom(room) {
        void room;
        return [];
    }
}
exports.InnerShipStatus = InnerShipStatus;
//# sourceMappingURL=InnerShipStatus.js.map